@@ -1,11 +1,11 @@
-import zmq
+import pyzmq
 from threading import Thread, Event
 import json
 import pandas as pd
 import os
 from time import sleep
 
-versionNo = "0.1.7"
+versionNo = "1.0.0"
 
 controlEndpoint = "tcp://127.0.0.1:5557"
 dataEndpoint = "tcp://127.0.0.1:5556"
@@ -56,12 +56,12 @@ def createControlMessage(details):
 
 def sendControl(controlDict):  
     controlString = createControlMessage(controlDict)
-    context = zmq.Context()
-    controlSock = context.socket(zmq.REQ)
+    context = pyzmq.Context()
+    controlSock = context.socket(pyzmq.REQ)
     controlSock.connect(controlEndpoint)
     controlSock.send(controlString)
     returnMessage = controlSock.recv_json()
-    controlSock.setsockopt(zmq.LINGER, 0)
+    controlSock.setsockopt(pyzmq.LINGER, 0)
     controlSock.close()
     context.term()
     if returnMessage["resultCode"] != "ok":
@@ -162,13 +162,21 @@ class heat():
 
 
     def set(self, setPoint):
+        self.data.setPointChanged()
         self.startRamp(setPoint, "rampTime", 0)
 
 class bias():
     def __init__(self):
         self.data = dataHandler("bias")
         self.busy = False
-
+        
+    def getConfiguration(self):
+        controlDict = {
+            '$type' : 'control.bias.getConfiguration'        
+            }
+        returnMessage = sendControl(controlDict)
+        return returnMessage
+    
     def set(self, setPoint, compliance):
         controlDict = {
             '$type' : 'control.bias.applyConstantBias',
@@ -176,6 +184,7 @@ class bias():
             'compliance' : compliance
             }
         returnMessage = sendControl(controlDict)
+        self.data.setPointChanged()
         return returnMessage["resultCode"]
     
     def startSweepCycle(self, baseline, amplitude, sweepCycleType, rate, cycles, compliance):
@@ -210,13 +219,12 @@ class gas():
         self.gasSystemType = None
         self.controlMode = None
         
-    def getSystemInfo(self):
+    def getConfiguration(self):
         controlDict = {
             '$type' : 'control.gas.getConfiguration'        
             }
         returnMessage = sendControl(controlDict)
-        self.gasSystemType = returnMessage["data"]["$type"].split(".")[-1]
-        self.controlMode = returnMessage["data"]["controlMode"]
+        self.gasSystemType = returnMessage['data']['$type'].split('.')[-1]
         return returnMessage
 
     def setBypass(self, toggleState):
@@ -262,7 +270,7 @@ class gas():
 
     def startIOPRamp(self, inletPressure, outletPressure, rampTime, gas1Flow=None, gas1FlowPath=None, gas2Flow=None, gas2FlowPath=None, gas3Flow=None, gas3FlowPath=None ):
         if self.gasSystemType is None:
-            self.getSystemInfo()
+            self.getConfiguration()
         controlDict = {
             '$type' : 'control.gas.' + self.gasSystemType + '.inletOutletPressure.startRamp',
             'inletPressure' : inletPressure,
@@ -283,11 +291,12 @@ class gas():
     
     def setIOP(self, inletPressure, outletPressure, gas1Flow=None, gas1FlowPath=None, gas2Flow=None, gas2FlowPath=None, gas3Flow=None, gas3FlowPath=None):
         returnMessage = self.startIOPRamp(inletPressure, outletPressure, 0, gas1Flow, gas1FlowPath, gas2Flow, gas2FlowPath, gas3Flow, gas3FlowPath)
+        self.data.setPointChanged()
         return returnMessage
     
     def startPFRamp(self, reactorPressure, reactorFlow, rampTime, gasConcentrationType=None, gas1Concentration=None, gas2Concentration=None ):
         if self.gasSystemType is None:
-            self.getSystemInfo()
+            self.getConfiguration()
         controlDict = {
             '$type' : 'control.gas.' + self.gasSystemType + '.pressureFlow.startRamp',
             'reactorPressure' : reactorPressure,
@@ -305,6 +314,7 @@ class gas():
     
     def setPF(self, reactorPressure, reactorFlow, gasConcentrationType=None, gas1Concentration=None, gas2Concentration=None):
         returnMessage = self.startPFRamp(reactorPressure, reactorFlow, 0, gasConcentrationType, gas1Concentration, gas2Concentration)
+        self.data.setPointChanged()
         return returnMessage
     
     def initiateFlow(self, reactorPressure, reactorFlow, gasConcentrationType, gas1Concentration, gas2Concentration):
@@ -333,40 +343,44 @@ class eventHandler():
         self.eventSock = None
 
     def subscribe(self):
-        self.context = zmq.Context()
-        self.eventSock = self.context.socket(zmq.SUB)
+        self.context = pyzmq.Context()
+        self.eventSock = self.context.socket(pyzmq.SUB)
         self.eventSock.connect(dataEndpoint)
         btopic = bytes('event', 'utf_8')
-        self.eventSock.setsockopt(zmq.SUBSCRIBE, btopic)
+        self.eventSock.setsockopt(pyzmq.SUBSCRIBE, btopic)
         self.eventCatcherThread = Thread(target=self.eventCatcher)
         self.eventCatcherThread.start()
 
     def eventCatcher(self):
-        print("Subscribed to impulse events")
+        print("Subscribed to impulse events. ")
         sleep(0.01)
         while True:
             if self.event.is_set():
                 break
-            [address, contents] = self.eventSock.recv_multipart()
-            contents = json.loads(contents)
-            self.lastMssg = contents
-            if contents["$type"]=="data.event.stimulusActionFinished":
-                if contents["stimulusType"]=="heat":
-                    heat.busy = False
-                if contents["stimulusType"]=="bias":
-                    bias.busy = False
-                if contents["stimulusType"]=="gas":
-                    gas.busy = False
-            elif contents["$type"] == "data.event.profileController.stateChanged": #deze moet ook anders heten
-                print("[PROFILE CONTROLLER]:",contents['state'])
-            elif contents["$type"] == "data.event.popupRaised":
-                print("[IMPULSE POPUP]:",contents['message'])
-            elif contents["$type"] == "data.event.notificationRaised":
-                print("[IMPULSE NOTIFICATION]:",contents['message'])
-                sleep(0.1)
+            try:
+                [address, contents] = self.eventSock.recv_multipart()
+            except:
+                continue
             else:
-                print("[IMPULSE EVENT]:",contents)
-        self.eventSock.setsockopt(zmq.LINGER, 0)
+                contents = json.loads(contents)
+                self.lastMssg = contents
+                if contents["$type"]=="data.event.stimulusActionFinished":
+                    if contents["stimulusType"]=="heat":
+                        heat.busy = False
+                    if contents["stimulusType"]=="bias":
+                        bias.busy = False
+                    if contents["stimulusType"]=="gas":
+                        gas.busy = False
+                elif contents["$type"] == "data.event.profileController.stateChanged": #deze moet ook anders heten
+                    print("[PROFILE CONTROLLER]:",contents['state'])
+                elif contents["$type"] == "data.event.popupRaised":
+                    print("[IMPULSE POPUP]:",contents['message'])
+                elif contents["$type"] == "data.event.notificationRaised":
+                    print("[IMPULSE NOTIFICATION]:",contents['message'])
+                    sleep(0.1)
+                else:
+                    print("[IMPULSE EVENT]:",contents)
+        self.eventSock.setsockopt(pyzmq.LINGER, 0)
         self.eventSock.close()
         self.context.term()
         self.eventSock = None # For jupyter notebook if a cell is rerun
@@ -390,50 +404,52 @@ class dataHandler():
         self.event = Event()
 
     def subscribe(self):
-        self.context = zmq.Context()
-        self.dataSock = self.context.socket(zmq.SUB)
+        self.context = pyzmq.Context()
+        self.dataSock = self.context.socket(pyzmq.SUB)
         self.dataSock.connect(dataEndpoint)
         btopic = bytes(self.topic, 'utf_8')
-        self.dataSock.setsockopt(zmq.SUBSCRIBE, btopic)
+        self.dataSock.setsockopt(pyzmq.SUBSCRIBE, btopic)
         self.dataThread = Thread(target=self.incomingDataReader)
         self.dataThread.start()
-        print(f"Subscribed to {self.topic} data")
+        print(f"Subscribed to {self.topic} data. ")
 
     def incomingDataReader(self):
         if self.topic != "massSpec":
             while True:
                 if self.event.is_set():
                     break
-                message = self.dataSock.recv_multipart()
-                if len(message)!=2:
-                    print("Not a multipart message!")
-                    print(message)
-                address = message[0]
-                contents = message [1]
-                contents = json.loads(contents)
-                contents["flags"]=self.currentFlag
-                self.lastMssg = contents
-                if self.header is None:
-                    self.header = list(contents.keys())
-                    self.data = pd.DataFrame(columns=self.header)
-                self.data = self.data.append(contents, ignore_index=True, sort=False)
+                try:
+                    [address, contents] = self.dataSock.recv_multipart()
+                except:
+                    continue
+                else:
+                    contents = json.loads(contents)
+                    contents["flags"]=self.currentFlag
+                    self.lastMssg = contents
+                    if self.header is None:
+                        self.header = list(contents.keys())
+                        self.data = pd.DataFrame(columns=self.header)
+                    self.data = self.data.append(contents, ignore_index=True, sort=False)
         else: # Special DataReceiver for MS data
             while True:
                 if self.event.is_set():
                     break
-                [address, contents] = self.dataSock.recv_multipart()
-                contents = json.loads(contents)
-                for channel in contents["channels"]:
-                   contents[channel["name"]]=channel["measuredValue"]
-                del contents["channels"]
-                contents["flags"]=self.currentFlag
-                self.lastMssg = contents
-                if self.header is None:
-                    self.header = list(contents.keys())
-                    self.data = pd.DataFrame(columns=self.header)
-                self.data = self.data.append(contents, ignore_index=True, sort=False)
-
-        self.dataSock.setsockopt(zmq.LINGER, 0)
+                try:
+                    [address, contents] = self.dataSock.recv_multipart()
+                except:
+                    continue
+                else:
+                    contents = json.loads(contents)
+                    for channel in contents["channels"]:
+                       contents[channel["name"]]=channel["measuredValue"]
+                    del contents["channels"]
+                    contents["flags"]=self.currentFlag
+                    self.lastMssg = contents
+                    if self.header is None:
+                        self.header = list(contents.keys())
+                        self.data = pd.DataFrame(columns=self.header)
+                    self.data = self.data.append(contents, ignore_index=True, sort=False)
+        self.dataSock.setsockopt(pyzmq.LINGER, 0)
         self.dataSock.close()
         self.context.term()
         self.event.clear()
@@ -442,6 +458,8 @@ class dataHandler():
     def disconnect(self):
         self.event.set()
 
+    def setPointChanged(self):
+        self.lastSentSequence = self.lastMssg["sequenceNumber"]
 
     def roundDataFrame(self, dataSet):       
         dataSet = dataSet.round(decimalVals)
@@ -457,7 +475,10 @@ class dataHandler():
             return dataSet
 
         else:
-            print(f"[ALERT] Please subscribe using impulse.{self.topic}.data.subscribe() first")
+            if self.topic == 'massSpec':
+                print("[ALERT] Please subscribe using impulse.gas.msdata.subscribe() first")
+            else:
+                print(f"[ALERT] Please subscribe using impulse.{self.topic}.data.subscribe() first")
             return []
 
     def getNewData(self):
@@ -498,7 +519,7 @@ class dataHandler():
                     return []
             df = self.data.iloc[startRowOrFlag:endRowOrFlag].copy()
             if len(df)>1:
-                df["timeStamp"] = pd.to_datetime(df['timeStamp']).dt.tz_convert(None)
+                df["timeStamp"] = pd.to_datetime(df['timeStamp'])#.dt.tz_convert(None)
             df = self.roundDataFrame(df)
             return df
         else:
